# Choices
CHOICE_STATUS = (
		('0', 'Inactive'),
		('1', 'Active')
	)


# default column names

COLUMN_NAME_ACTIVE = 'active'
COLUMN_NAME_DELETED = 'deleted'

COLUMN_NAME_DELETED_BY = 'deleted_by'
COLUMN_NAME_DELETED_AT = 'deleted_at'

COLUMN_NAME_CREATED_BY = 'created_by'
COLUMN_NAME_CREATED_AT = 'created_at'

COLUMN_NAME_UPDATED_BY = 'updated_by'
COLUMN_NAME_UPDATED_AT = 'updated_at'





class MsbDatabaseRouter:
	__app_labels: list = []
	_app_relations: list = __app_labels
	_db_for_read: str = 'default'
	_db_for_write: str = 'default'
	db_mapping: dict = {
		"default": dict(read="default", write="default")
	}

	"""Determine how to route database calls for an app's models (in this case, for an app named 
	Example). All other models will be routed to the next router in the DATABASE_ROUTERS setting 
	if applicable, or otherwise to the default data_base. """

	def db_for_read(self, model):
		"""Send all read operations on Example app models to `example_db`."""
		if model._meta.app_label in self.__app_labels:
			return self.__db_for_read(app_label=model._meta.app_label)
		return None

	def db_for_write(self, model):
		"""Send all write operations on Example app models to `example_db`."""
		if model._meta.app_label in self.__app_labels:
			return self.__db_for_write(app_label=model._meta.app_label)
		return None

	def allow_relation(self, obj1, obj2):
		"""Determine if relationship is allowed between two objects."""

		# Allow any relation between two models that are both in the Example app.
		if obj1._meta.app_label in self._app_relations and obj2._meta.app_label in self._app_relations:
			return True
		# No opinion if neither object is in the Example app (defer to default or other routers).
		elif obj1._meta.app_label not in self._app_relations and obj2._meta.app_label not in self._app_relations:
			return None

		# Block relationship if one object is in the Example app and the other isn't.
		return False

	def allow_migrate(self, db, app_label, model_name=None):
		"""Ensure that the Example app's models get created on the right data_base."""
		if app_label in self.__app_labels:
			# The Example app should be migrated only on the example_db data_base.
			return db == self._db_for_write
		elif db == '':
			# Ensure that all other apps don't get migrated on the example_db data_base.
			return False

		# No opinion for all other scenarios
		return None

	def __init__(self):
		self.__app_labels = self.db_mapping.keys()

	def __db_for_read(self, app_label) -> str:
		return self.db_mapping.get(app_label).get("read")

	def __db_for_write(self, app_label) -> str:
		return self.db_mapping.get(app_label).get("write")



import logging

from ._constants import (SIMPLE_LOG_FORMAT)


# custom log handler that emits to the database
class DatabaseLogger(logging.Handler):
	_db_model = None

	def __init__(self, db_model=None, *args, **kwargs):
		logging.Handler.__init__(self, *args, **kwargs)
		self.setFormatter(logging.Formatter(SIMPLE_LOG_FORMAT))
		self.level = logging.INFO
		self._db_model = db_model

	def emit(self, record):
		try:
			if self._db_model:
				log = self._db_model(**record.__dict__)
				log.save()
		except Exception:
			pass



import logging

from django.db.models import QuerySet
from rest_framework import (viewsets)
from msb_core.auth import TokenUser

from ..wrappers import (ApiResponse, RequestInfo, RequestHeaders)
from msb_core.wrappers.validation import (Validate, InvalidPayloadException)


def api_details(request=None, ver='', name=''):
	return ApiResponse.success(
		data=dict(method=request.method, version=ver, name=name)
	)


class ApiView(viewsets.GenericViewSet):
	permission_classes = ()
	serializer_class = ''

	"""
	TODO : implement following code :
	
	@staticmethod
	@renderer_classes((JSONRenderer))
	def my_exception_handler(exc, context):
		# response = exception_handler(exc, context)  # <-- this is the default exception_handler
		import logging
		logging.exception(exc)
		return ApiResponse.internal_server_error()

	def get_exception_handler(self):
		return self.my_exception_handler
	"""

	def validate_data(self, rule: None, data=None):
		validation_errors = Validate.raw_schema(rules=rule, data=data)
		if validation_errors:
			raise InvalidPayloadException(errors=validation_errors)
		return data, validation_errors

	@property
	def request_info(self) -> RequestInfo:
		return RequestInfo(meta=self.request.META)

	@property
	def request_headers(self) -> RequestHeaders:
		return RequestHeaders(headers=self.request.headers)

	@property
	def user(self) -> TokenUser:
		return self.request.user

	@property
	def payload(self) -> dict:
		return self.request.data

	@property
	def logger(self):
		return logging.getLogger('root')

	@property
	def api_response(self):
		return ApiResponse

	def serializer(self, data: list = None):
		return (
			[item.dict() if hasattr(item, 'dict') else item for item in data]
		) if (type(data) in [list, QuerySet]) else []



from msb_core.exceptions import ApiException



class InvalidPayloadException(ApiException):
	_message = 'Invalid Request Data.'


class InvalidParamsException(ApiException):
	_message = 'Invalid Request Parameters.'



import logging

CHOICES_LOG_LEVELS = (
	(logging.INFO, 'INFO'),
	(logging.DEBUG, 'DEBUG'),
	(logging.WARNING, 'WARNING'),
	(logging.ERROR, 'ERROR'),
	(logging.CRITICAL, 'CRITICAL'),
)

LOG_DB = "logs"
LOG_TABLE = "system_logs"

DEBUG_REQUIRED_FILTER_CLASS = "django.utils.log.RequireDebugTrue"
DEBUG_REQUIRED_FILTER_NAME = 'require_debug_true'

SIMPLE_LOG_FORMAT = '[%(levelname)s] : %(message)s '
SIMPLE_LOG_FILE_FORMAT = '[%(levelname)s] %(asctime)s : %(message)s '
VERBOSE_LOG_FORMAT = "%(levelname)s %(asctime)s %(module)s %(process:d)s %(thread:d)s %(message)s "

LOG_DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"

LOG_FILTERS = {
	DEBUG_REQUIRED_FILTER_NAME: {"()": DEBUG_REQUIRED_FILTER_CLASS}
}




CRUD_URL_PK = '/<str:pk>'
CREATE_ACTION = 'create'
RETRIEVE_ACTION = 'retrieve'
UPDATE_ACTION = 'update'
DELETE_ACTION = 'delete'
LIST_ACTION = 'list'
LIST_URL = '/list'

ALL_ACTIONS = [
	CREATE_ACTION, RETRIEVE_ACTION,
	DELETE_ACTION, UPDATE_ACTION, LIST_ACTION
]



from functools import wraps

import rest_framework.request

from ._exceptions import (InvalidPayloadException, InvalidParamsException)
from msb_core.api.request import ApiResponse


class Validate:

	@staticmethod
	def raw_schema(rules: dict = None, data=None, *args, **kwargs):
		if not isinstance(rules, dict) or data is None:
			return

		allow_unknown = kwargs.get('unknown') == True
		from cerberus.validator import Validator
		_validator = Validator(rules, allow_unknown=allow_unknown)
		_validator.validate(data)
		errors = _validator.errors
		return errors if len(errors.keys()) > 0 else None

	@staticmethod
	def request_data(rule: dict = None, params: dict = None, **opt):
		def outer_func(_func):
			@wraps(_func)
			def inner_func(cls, request: rest_framework.request.Request, *args, **kwargs):
				try:

					# validate paload data if payload rules are defined
					payload_validation_errors = Validate.raw_schema(rules=rule, data=request.data)
					if payload_validation_errors:
						raise InvalidPayloadException(errors=payload_validation_errors)

					# validate parameter data like from URL/Query string if parameter
					# validation rules are defined
					params_validation_errors = Validate.raw_schema(rules=params, data=kwargs)
					if params_validation_errors:
						raise InvalidParamsException(errors=params_validation_errors)

					return _func(cls, *args, **dict(request=request, **kwargs))

				except InvalidPayloadException as e:
					return ApiResponse.exception(e)

				except InvalidParamsException as e:
					return ApiResponse.exception(e)

				except Exception as e:
					return ApiResponse.exception(e)

			return inner_func

		return outer_func

	@staticmethod
	def access(group: list = None, action: str = '', **opt):
		def decorator_function(_decorated_function):
			@wraps(_decorated_function)
			def wrapper_function(cls, request: rest_framework.request.Request, *args, **kwargs):
				try:
					return _decorated_function(cls, *args, **dict(request=request, **kwargs))
				except Exception as e:
					ApiResponse.error(e)

			return wrapper_function

		return decorator_function



import logging

from django.db.models import (manager, Model, DateTimeField, IntegerField, CharField, TextField)

from msb_core.conf.logging import (LOG_DB, LOG_TABLE, CHOICES_LOG_LEVELS)


class LoggingModelManager(manager.BaseManager):
	log_db = LOG_DB

	def get_queryset(self):
		return super().get_queryset().using(self.log_db)


class LoggingModel(Model):
	class Meta:
		abstract = True
		managed = False

	level = CharField(choices=CHOICES_LOG_LEVELS, max_length=255, default=logging.INFO)
	message = TextField(default=None, null=True)
	created_at = DateTimeField(auto_now_add=True, db_column='created_at')


class SystemLogModel(LoggingModel):
	class Meta:
		db_table = LOG_TABLE
		abstract = True
		managed = False

	logger = CharField(max_length=255, default='root')

	msg = TextField(default=None)
	module = CharField(max_length=255, default=None, null=True)
	func_name = CharField(max_length=255, default=None, null=True)
	thread_name = CharField(max_length=255, default=None, null=True)
	process_name = CharField(max_length=255, default=None, null=True)
	filename = CharField(max_length=255, default=None, null=True)

	args = TextField(default=None, null=True)
	pathname = TextField(default=None, null=True)
	exc_info = TextField(default=None, null=True)
	exc_text = TextField(default=None, null=True)
	stack_info = TextField(default=None, null=True)
	lineno = IntegerField(default=0, null=True)

	objects = LoggingModelManager()

	def __init__(self, *args, **logdetails):
		super(SystemLogModel, self).__init__(*args, **{})

		self.logger = logdetails.get('name')
		self.level = logdetails.get('levelname')
		self.msg = logdetails.get('msg')
		self.message = logdetails.get('message')
		self.module = logdetails.get('module')
		self.filename = logdetails.get('filename')

		self.func_name = logdetails.get('funcName')
		self.thread_name = logdetails.get('threadName')
		self.process_name = logdetails.get('processName')
		self.args = logdetails.get('args')
		self.pathname = logdetails.get('pathname')
		self.exc_info = logdetails.get('exc_info')
		self.exc_text = logdetails.get('exc_text')
		self.stack_info = logdetails.get('stack_info')
		self.lineno = logdetails.get('lineno')



import datetime

from django.core.serializers import serialize
from django.db import models
from django.db import models as Models
from .._constants import (COLUMN_NAME_DELETED, COLUMN_NAME_DELETED_BY, COLUMN_NAME_CREATED_AT, COLUMN_NAME_UPDATED_AT,
                          COLUMN_NAME_CREATED_BY, COLUMN_NAME_UPDATED_BY, COLUMN_NAME_ACTIVE, COLUMN_NAME_DELETED_AT)
from msb_core.wrappers.encryption import Cipher


class MsbModelManager(models.Manager):
	_default_filters = {}

	def _get_filtered_queryset(self, **filters):
		_query_set = super(MsbModelManager, self).get_queryset()
		query_filters = {}
		for filtername, filtervalue in filters.items():
			if hasattr(self.model, filtername):
				query_filters[filtername] = filtervalue
		return _query_set.filter(**query_filters)

	def get(self, *args, **kwargs):
		filters = {**kwargs}
		pk = kwargs.get('pk')
		if pk is not None and isinstance(pk, str):
			filters['pk'] = int(Cipher.decrypt(pk))
		return super(MsbModelManager, self).get(*args, **filters)

	@property
	def deleted(self):
		return self._get_filtered_queryset(**{COLUMN_NAME_DELETED: True})

	def get_queryset(self):
		return self._get_filtered_queryset(**{COLUMN_NAME_DELETED: False, **self._default_filters})


class MsbModel(Models.Model):
	_private_fields = []
	_identifier_fields = 'id'

	class Meta:
		abstract = True

	@property
	def related_fields(self):
		fields = []
		for field in self._meta.fields:
			if field.get_internal_type() in ['ForeignKey']:
				fields.append(field.name)
		return fields

	def dict(self, encrypted=True):
		try:
			return {
				k: v if (k not in [
					self._meta.pk.attname, *self._private_fields
				] or not encrypted) else Cipher.encrypt(v)
				for k, v in super().__dict__.items()
				if not k.startswith('__') and not k.startswith('_') and not callable(k)
			}

		except Exception:
			return dict()

	@property
	def serialized(self):
		return serialize('python', [self])

	def current_timestamp(self):
		return datetime.datetime.now()

	def delete(self, deleted_by=None, using=None, keep_parents=False):
		if hasattr(self, COLUMN_NAME_DELETED):
			self.__setattr__(COLUMN_NAME_DELETED, True)

		if hasattr(self, COLUMN_NAME_DELETED_BY):
			self.__setattr__(COLUMN_NAME_DELETED_BY, deleted_by)

		self.save()
		return True

	def __str__(self):
		return f"< {self.__class__.__name__}, pk[{self.pk}]  >"

	def __unicode__(self):
		return self.__str__()


class MsbMetaModel(MsbModel):
	updated_at = models.DateTimeField(db_column=COLUMN_NAME_UPDATED_AT, auto_now=True)
	updated_by = models.IntegerField(db_column=COLUMN_NAME_UPDATED_BY, null=True, default=None)
	created_at = models.DateTimeField(db_column=COLUMN_NAME_CREATED_AT, auto_now_add=True)
	created_by = models.IntegerField(db_column=COLUMN_NAME_CREATED_BY, null=True, default=None)
	is_deleted = models.BooleanField(db_column=COLUMN_NAME_DELETED, default=False, null=False)



from msb_core.classes import Singleton
from msb_core.exceptions import CustomExceptionHandler,AppException
from msb_core.wrappers.encryption import Cipher


class ApiService(CustomExceptionHandler, metaclass=Singleton):
	__error_msg = 'BaseService.db_model not found.'
	db_model = None

	@property
	def cipher(self):
		return Cipher

	def create(self, **model_data):
		try:
			if self.db_model is None:
				raise AppException(self.__error_msg)

			model = self.db_model(**model_data)
			model.save()
			return model
		except Exception as e:
			return False

	def update(self, pk=None, **model_data):
		try:
			if self.db_model is None:
				raise AppException(self.__error_msg)
			res = self.db_model.objects.filter(pk=pk).update(**model_data)
			if not res:
				raise Exception
			return True
		except Exception as e:
			raise Exception('Update Operation Failed')

	def retrieve(self, pk=None):
		try:
			if self.db_model is None:
				raise AppException(self.__error_msg)
			return self.db_model.objects.get(pk=pk)
		except Exception:
			return False

	def delete(self, pk=None):
		try:
			if self.db_model is None:
				raise AppException(self.__error_msg)
			status = self.db_model.objects.get(pk=pk).delete()
			if not status:
				raise Exception
			return status
		except Exception as e:
			raise Exception('Delete Operation Failed')

	def list(self, fields: list = None, filters: dict = None):
		try:
			if self.db_model is None:
				raise AppException(self.__error_msg)
			fields = fields if type(fields) == list else []
			filters = filters if type(filters) == dict else dict()
			data_list = self.db_model.objects.only(*fields).filter(**filters).all()
			return data_list
		except Exception as e:
			self.handle_exception(e)
			return False



from . import _constants as API_CONST
from msb_core.wrappers.validation import DefaultRules, Validate

from ..wrappers import (ApiResponse, RestResponse)
from ..services import ApiService
from ._api_view import ApiView





class ApiViewset(ApiView):
	service_class: ApiService = None
	validation_rules = None
	allowed_crud = []

	@classmethod
	def crud_routes(cls, actions=None, **initkwargs):
		from django.urls import path
		return [
			path("", cls.as_view(actions={"post": "create"})),
			path("/list", cls.as_view(actions={"get": "list"})),
			path(
				"/<str:pk>",
				cls.as_view({"get": "retrieve", "put": "update", "delete": "delete"})
			),
		]

	@Validate.request_data(params=dict(pk=DefaultRules.pk.schema))
	def retrieve(self, *args, **kwargs) -> RestResponse:
		if self.service_class is None or API_CONST.RETRIEVE_ACTION not in self.allowed_crud:
			return self.api_response.method_not_allowed()
		try:
			pk = kwargs.get('pk')
			result = self.service_class.retrieve(pk=pk)
			return self.api_response.success(data=self.serializer(data=[result]))
		except Exception as e:
			return self.api_response.exception(e=e)

	@Validate.request_data(params=dict(pk=DefaultRules.pk.schema))
	def delete(self, *args, **kwargs) -> RestResponse:
		if self.service_class is None or API_CONST.DELETE_ACTION not in self.allowed_crud:
			return self.api_response.method_not_allowed()
		try:
			pk = kwargs.get('pk')
			result = self.service_class.delete(pk=pk)
			return self.api_response.success() if result else self.api_response.error()
		except Exception as e:
			return self.api_response.exception(e=e)

	def create(self, *args, **kwargs) -> RestResponse:
		if self.service_class is None or API_CONST.CREATE_ACTION not in self.allowed_crud:
			return self.api_response.method_not_allowed()
		try:
			rule = self.validation_rules.create if \
				hasattr(self.validation_rules, API_CONST.CREATE_ACTION) else {}
			data, *_ = self.validate_data(rule=rule, data=self.payload)
			data['created_by'] = self.user.id
			result = self.service_class.create(**data)
			return self.api_response.success() if result else self.api_response.error()
		except Exception as e:
			return self.api_response.exception(e)

	@Validate.request_data(params=dict(pk=DefaultRules.pk.schema))
	def update(self, *args, **kwargs) -> RestResponse:
		if self.service_class is None or API_CONST.UPDATE_ACTION not in self.allowed_crud:
			return self.api_response.method_not_allowed()
		try:
			pk = kwargs.get('pk')
			rule = self.validation_rules.update if \
				hasattr(self.validation_rules, API_CONST.UPDATE_ACTION) else {}
			data, *_ = self.validate_data(rule=rule, data=self.payload)
			data['updated_by'] = self.user.id
			res = self.service_class.update(pk=pk, **data)
			if not res:
				raise Exception
			return self.api_response.success()
		except Exception as e:
			return self.api_response.exception(e=e)

	@Validate.request_data(params=None)
	def list(self, *args, **kwargs) -> RestResponse:
		if self.service_class is None or API_CONST.LIST_ACTION not in self.allowed_crud:
			return self.api_response.method_not_allowed()
		try:
			requested_fields = self.payload.get('fields')
			requested_filters = self.payload.get('filters')
			result = self.service_class.list(fields=requested_fields, filters=requested_filters)
			return self.api_response.success(data=self.serializer(data=result))
		except Exception as e:
			return self.api_response.exception(e=e)



from msb_core.exceptions import AppException
from msb_core.db.models import MsbModel
from dataclasses import dataclass


@dataclass
class SearchParameter:
	def __init__(self):
		self._filters = dict()
		self._fields = []
		self._order_by = 'ASC'
		self._limit = 50
		self._offset = 0
		self._order_field = None
		self._db_model = None

	@property
	def filters(self) -> dict:
		return self._filters

	@property
	def fields(self) -> list:
		return self._fields

	@property
	def order_by(self) -> str:
		return self._order_by

	@property
	def limit(self) -> int:
		return self._limit

	@property
	def offset(self) -> int:
		return self._offset

	@property
	def order_field(self) -> str:
		return self._order_field

	@property
	def db_model(self) -> MsbModel:
		return self._db_model

	@filters.setter
	def filters(self, value: dict):
		if value is not None:
			self._filters = value

	@fields.setter
	def fields(self, value: list):
		if value is not None:
			self._fields = value

	@order_by.setter
	def order_by(self, value: str):
		if value is not None:
			self._order_by = value

	@limit.setter
	def limit(self, value: int):
		if value is not None:
			self._limit = value

	@offset.setter
	def offset(self, value: int):
		if value is not None:
			self._offset = value

	@order_field.setter
	def order_field(self, value: str):
		if value is not None:
			self._order_field = value

	@db_model.setter
	def db_model(self, value: MsbModel):
		self._db_model = value

	@property
	def search(self):
		try:
			if self.db_model is None:
				raise AppException(self.__error_msg)
			return self._db_model.objects.only(*self._fields).filter(**self._filters).all()[
			       self._offset:self._offset + self._limit]
		except Exception as e:
			self.handle_exception(e)
			return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['msb_core',
 'msb_core.api',
 'msb_core.api.services',
 'msb_core.api.views',
 'msb_core.api.wrappers',
 'msb_core.auth',
 'msb_core.classes',
 'msb_core.conf',
 'msb_core.conf.env',
 'msb_core.conf.logging',
 'msb_core.db',
 'msb_core.db.models',
 'msb_core.devscripts',
 'msb_core.exceptions',
 'msb_core.testing',
 'msb_core.wrappers',
 'msb_core.wrappers.datetime',
 'msb_core.wrappers.encryption',
 'msb_core.wrappers.validation',
 'msb_ext']

package_data = \
{'': ['*']}

install_requires = \
['cerberus>=1.3.4,<2.0.0',
 'cryptography>=38.0.3,<39.0.0',
 'django>=4.1.3,<5.0.0',
 'djangorestframework-simplejwt==5.1.0',
 'djangorestframework==3.13.1',
 'python-dateutil>=2.8.2,<3.0.0',
 'python-dotenv>=0.21.0,<0.22.0',
 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'hrin-msb',
    'version': '0.1.0',
    'description': '',
    'long_description': '# hrin-msb\n\n## Pre-requisites for setup\n1. `pip install poetry`\n\n## How To Build\n\n1. `poetry build`\n2. `poetry publish`',
    'author': 'Prakash Mishra',
    'author_email': 'prakash.mishra@intimetec.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from ._rules import (DefaultRules)
from ._schema import (RuleSchema, InputField)
from ._validators import (Validate)
from ._exceptions import (InvalidPayloadException, InvalidParamsException)

__all__ = [
	"DefaultRules", "RuleSchema", "InputField", "Validate",
	"InvalidParamsException", "InvalidPayloadException"
]



class AppException(Exception):
	_message: str = ''
	_errors: list = []
	_code: int = 600

	def __init__(self, message: str = None, errors: list = None):
		super().__init__(message if message is not None else self._message)
		self._errors = errors if isinstance(errors, dict) else list()

	@property
	def message(self):
		return self._message

	@property
	def errors(self):
		return self._errors

	@property
	def code(self):
		return self._code

	def log(self):
		import logging
		logging.exception(self)


class ApiException(AppException):
	def __init__(self, message: str = None, errors: dict = None):
		super().__init__(message=message, errors=errors)



import logging
import os
from datetime import datetime

from ._constants import (LOG_DATETIME_FORMAT)


def make_log_file_name(dir: str = '', filename: str = None):
	filename = filename if filename not in ['', None] else f"{datetime.today().strftime('%Y-%m-%d')}.log"
	return os.path.join(dir, filename)


class LogConfig:
	_registered_loggers: list
	_registered_handlers: list
	_default_log_level: str
	_default_format_str: str

	def __init__(self, **kwargs):
		self._registered_loggers = list()
		self._registered_handlers = list()
		self._default_log_level = kwargs.get('default_level') or logging.WARNING

	def add_formatter(self, name: str, format: str = '', style: str = None):
		self.formatters[name] = {"format": format}

	def add_filters(self, name: str, handler_class: str, **kwargs):
		self.filters[name] = {
			'()': 'django.utils.log.RequireDebugTrue',
		}

	def add_logger(self, name: str, level: str = None, handlers: list = None, filters: list = None, **kwargs):
		try:
			log_handlers = handlers if isinstance(handlers, list) else []
			_logger = logging.getLogger(name=name)
			log_level = level if level is not None else self._default_log_level

			_logger.setLevel(level=log_level)
			_logger.propagate = kwargs.get('propagate') == True

			for log_handler in log_handlers:
				_logger.addHandler(log_handler)

			self._registered_loggers.append(name)
		except Exception as e:
			print(f"Logger registration failed for '{name}' :\n{e}")

	def get_file_handler(self, name, logs_dir: str, filename: str = None, handler_class=None, **kwargs):
		from logging.handlers import TimedRotatingFileHandler
		handler_class = handler_class if callable(handler_class) else TimedRotatingFileHandler
		init_kwargs = dict(filename=make_log_file_name(dir=logs_dir, filename=filename), when='midnight')
		return self.__build_log_handler(
			name=name, log_handler=handler_class, init_kwargs=init_kwargs, level=kwargs.get('level'),
			formatter=kwargs.get("formatter"), filters=kwargs.get('filters')
		)

	def get_email_handler(self, name, email_backend, handler_class: str = None, **kwargs):
		from django.utils.log import AdminEmailHandler
		handler_class = handler_class if callable(handler_class) else AdminEmailHandler
		init_kwargs = dict(
			include_html=kwargs.get('include_html') == True,
			email_backend=email_backend,
			reporter_class=kwargs.get('reporter_class')
		)
		return self.__build_log_handler(
			name=name, log_handler=handler_class, init_kwargs=init_kwargs, level=kwargs.get('level'),
			formatter=kwargs.get("formatter"), filters=kwargs.get('filters')
		)

	def get_console_handler(self, name, handler_class: str = None, **kwargs):
		handler_class = handler_class if callable(handler_class) else logging.StreamHandler
		init_kwargs = {}
		return self.__build_log_handler(
			name=name, log_handler=handler_class, init_kwargs=init_kwargs,
			level=kwargs.get('level'), formatter=kwargs.get("formatter"), filters=kwargs.get('filters')
		)

	def get_database_handler(self, name, db_model, **kwargs):
		init_kwargs = dict(db_model=db_model)
		return self.__build_log_handler(
			name=name, log_handler=DatabaseLogger, init_kwargs=init_kwargs,
			level=kwargs.get('level'), formatter=kwargs.get("formatter"), filters=kwargs.get('filters')
		)

	def __build_log_handler(self, name, log_handler: logging.Handler, init_kwargs: dict = None, **kwargs):
		init_kwargs = init_kwargs if isinstance(init_kwargs, dict) else {}
		log_filters = kwargs.get("filters") or []
		log_formatter = kwargs.get("formatter") or self._default_format_str
		log_level = kwargs.get("level") or self._default_log_level

		_handler = log_handler(**init_kwargs)
		_handler.setLevel(level=log_level)
		_handler.setFormatter(logging.Formatter(log_formatter, datefmt=LOG_DATETIME_FORMAT))
		for log_filter in log_filters:
			_handler.addFilter(log_filter)
		self._registered_handlers.append(name)
		return _handler



from .models import *
from ._constants import *
from ._fields import (
	EncryptedBool, EncryptedInteger, EncryptedString,
	EncryptedFloat, EncryptedPrimaryKey
)
from ._routers import MsbDatabaseRouter

__all__ = [
	'Configuration', 'ConfigurationModelManager', 'MsbModel', 'MsbModelManager',
	'EncryptedBool', 'EncryptedInteger', 'EncryptedString', 'EncryptedFloat', 'EncryptedPrimaryKey',
	'LoggingModelManager', 'SystemLogModel',
	"MsbDatabaseRouter"

]
